"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpAdd = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
class OpAdd extends AbstractOp_1.AbstractOp {
    constructor(path, value) {
        super('add', path);
        this.value = value;
    }
    apply(doc) {
        const ref = json_pointer_1.find(doc, this.path);
        if (json_pointer_1.isObjectReference(ref))
            ref.obj[ref.key] = this.value;
        else if (json_pointer_1.isArrayReference(ref)) {
            if (ref.key > ref.obj.length)
                throw new Error('OUT_OF_BOUNDS');
            if (ref.key === ref.obj.length)
                ref.obj.push(this.value);
            else
                ref.obj.splice(ref.key, 0, this.value);
        }
        else
            doc = this.value;
        return { doc, old: ref.val };
    }
    toJson() {
        return {
            op: this.op,
            path: json_pointer_1.formatJsonPointer(this.path),
            value: this.value,
        };
    }
    toPacked() {
        return [0, this.path, { v: this.value }];
    }
}
exports.OpAdd = OpAdd;
