import { Observable } from 'rxjs';
import { assertId, isArray } from './util';
export class JsonRxClient {
    constructor({ send }) {
        this.cnt = 1;
        this.observers = new Map();
        this.send = send;
    }
    sendSubscribe(message) {
        this.send(message);
    }
    sendUnsubscribe(id) {
        this.send([-3, id]);
    }
    onData(message) {
        const [, id, payload] = message;
        assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.next(payload);
    }
    onComplete(message) {
        const [, id, payload] = message;
        assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        if (payload !== undefined)
            observer.observer.next(payload);
        observer.observer.complete();
    }
    onError(message) {
        const [, id, error] = message;
        assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.error(error);
    }
    onMessage(message) {
        if (!isArray(message))
            throw new Error('Invalid message');
        const [type] = message;
        if (type === 0)
            return this.onComplete(message);
        if (type === -1)
            return this.onError(message);
        if (type === -2)
            return this.onData(message);
        throw new Error('Invalid message');
    }
    call(method, payload) {
        const id = this.cnt++;
        const observable = new Observable((observer) => {
            const entry = { observer };
            this.observers.set(id, entry);
            return () => {
                this.observers.delete(id);
                if (!entry.completed)
                    this.sendUnsubscribe(id);
            };
        });
        this.sendSubscribe([id, method, payload]);
        return observable;
    }
    notify(name, payload) {
        const message = payload !== undefined ? [name, payload] : [name];
        this.send(message);
    }
}
