"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasOwnProperty = exports.isValidIndex = exports.parent = exports.isRoot = exports.isPathEqual = exports.isChild = exports.toPath = exports.formatJsonPointer = exports.parseJsonPointer = exports.escapeComponent = exports.unescapeComponent = void 0;
function unescapeComponent(component) {
    return component.replace(/~1/g, '/').replace(/~0/g, '~');
}
exports.unescapeComponent = unescapeComponent;
function escapeComponent(component) {
    if (component.indexOf('/') === -1 && component.indexOf('~') === -1)
        return component;
    return component.replace(/~/g, '~0').replace(/\//g, '~1');
}
exports.escapeComponent = escapeComponent;
function parseJsonPointer(pointer) {
    if (!pointer)
        return [];
    return pointer.slice(1).split('/').map(unescapeComponent);
}
exports.parseJsonPointer = parseJsonPointer;
function formatJsonPointer(path) {
    if (exports.isRoot(path))
        return '';
    return '/' + path.map(function (component) { return escapeComponent(String(component)); }).join('/');
}
exports.formatJsonPointer = formatJsonPointer;
exports.toPath = function (pointer) { return (typeof pointer === 'string' ? parseJsonPointer(pointer) : pointer); };
function isChild(parent, child) {
    if (parent.length >= child.length)
        return false;
    for (var i = 0; i < parent.length; i++)
        if (parent[i] !== child[i])
            return false;
    return true;
}
exports.isChild = isChild;
function isPathEqual(p1, p2) {
    if (p1.length !== p2.length)
        return false;
    for (var i = 0; i < p1.length; i++)
        if (p1[i] !== p2[i])
            return false;
    return true;
}
exports.isPathEqual = isPathEqual;
exports.isRoot = function (path) { return !path.length; };
function parent(path) {
    if (path.length < 1)
        throw new Error('NO_PARENT');
    return path.slice(0, path.length - 1);
}
exports.parent = parent;
function isValidIndex(index) {
    if (typeof index === 'number')
        return true;
    var n = parseInt(index, 10);
    return String(n) === index && n >= 0;
}
exports.isValidIndex = isValidIndex;
var Object$prototype$hasOwnProperty = Object.prototype.hasOwnProperty;
function hasOwnProperty(obj, key) {
    return Object$prototype$hasOwnProperty.call(obj, key);
}
exports.hasOwnProperty = hasOwnProperty;
