"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxClient = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var util_1 = require("./util");
var JsonRxClient = (function () {
    function JsonRxClient(_a) {
        var send = _a.send;
        this.cnt = 1;
        this.observers = new Map();
        this.send = send;
    }
    JsonRxClient.prototype.sendSubscribe = function (message) {
        this.send(message);
    };
    JsonRxClient.prototype.sendUnsubscribe = function (id) {
        this.send([-3, id]);
    };
    JsonRxClient.prototype.onData = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], payload = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.next(payload);
    };
    JsonRxClient.prototype.onComplete = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], payload = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        if (payload !== undefined)
            observer.observer.next(payload);
        observer.observer.complete();
    };
    JsonRxClient.prototype.onError = function (message) {
        var _a = tslib_1.__read(message, 3), id = _a[1], error = _a[2];
        util_1.assertId(id);
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.error(error);
    };
    JsonRxClient.prototype.onMessage = function (message) {
        if (!util_1.isArray(message))
            throw new Error('Invalid message');
        var _a = tslib_1.__read(message, 1), type = _a[0];
        if (type === 0)
            return this.onComplete(message);
        if (type === -1)
            return this.onError(message);
        if (type === -2)
            return this.onData(message);
        throw new Error('Invalid message');
    };
    JsonRxClient.prototype.call = function (method, payload) {
        var _this = this;
        var id = this.cnt++;
        var observable = new rxjs_1.Observable(function (observer) {
            var entry = { observer: observer };
            _this.observers.set(id, entry);
            return function () {
                _this.observers.delete(id);
                if (!entry.completed)
                    _this.sendUnsubscribe(id);
            };
        });
        this.sendSubscribe([id, method, payload]);
        return observable;
    };
    JsonRxClient.prototype.notify = function (name, payload) {
        var message = payload !== undefined ? [name, payload] : [name];
        this.send(message);
    };
    return JsonRxClient;
}());
exports.JsonRxClient = JsonRxClient;
